// Copyright (c) 2023 VMware Carbon Black, all rights reserved

// Package controllers contains classes dealing with the actual control functions behind the routes.
package controllers

import (
	"hello-api/internal/config"
	"hello-api/internal/metrics"

	"hello-api/internal/app/middleware"
	"hello-api/internal/app/parsers/deviceresponseheader"

	"github.com/gin-gonic/gin"
)

const (
	// HelloResponseCode -- As per: https://gitlab.bit9.local/psc/platform/ems/hinting/-/blob/main/pkg/hints/messagecode/message_code.go
	HelloResponseCode = 0x201d
)

type HelloController struct {
	deregHandler  gin.HandlerFunc
	helloHandler  gin.HandlerFunc
	preregHandler gin.HandlerFunc
}

// NewHelloController -- builder method for hello controller
func NewHelloController(conf *config.Config, metricsClient metrics.Service) HelloController {
	return HelloController{helloHandler: newHelloHandler(deviceresponseheader.NewResponseHeaderEncoder()), deregHandler: newDeregHandler(conf, metricsClient), preregHandler: newPreregHandler(metricsClient)}
}

type postRouterGroup interface {
	POST(string, ...gin.HandlerFunc) gin.IRoutes
}

func (controller *HelloController) AddToRouter(routerGroup postRouterGroup, lctPublisherMiddleware, hintingMiddleware gin.HandlerFunc) {
	routerGroup.POST("/hello", middleware.DeviceRequestHeaderMiddleware, controller.preregHandler, middleware.DeviceContextMiddleware, controller.deregHandler, lctPublisherMiddleware, hintingMiddleware, controller.helloHandler)
}
