package controllers

import (
	"fmt"
	"net/http"

	"hello-api/internal/app/middleware"
	"hello-api/internal/app/parsers/deviceresponseheader"
	"hello-api/internal/utils"

	"github.com/gin-gonic/gin"
	"gitlab.bit9.local/psc/ems/hinting/pkg/hints"
	"go.uber.org/zap"
)

type helloHandler struct {
	responseBodyEncoder deviceresponseheader.ResponseHeaderEncoder
}

func newHelloHandler(responseBodyEncoder deviceresponseheader.ResponseHeaderEncoder) gin.HandlerFunc {
	handler := helloHandler{responseBodyEncoder: responseBodyEncoder}
	return handler.doHello
}

func (controller helloHandler) doHello(c *gin.Context) {
	hintResult, err := utils.ContextGet[*hints.HintResult](c, middleware.HintsName)
	if err != nil {
		_ = c.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to get hints from context: %w", err))
		return
	}
	version, err := utils.ContextGet[int](c, middleware.VersionName)
	if err != nil {
		_ = c.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to get version from context: %w", err))
		return
	}

	hintsArray := make([]int, 0)
	if hintResult.Hints != nil {
		hintsArray = hintResult.Hints.ToIntArray()
	}
	header := deviceresponseheader.NewDeviceResponseHeader(int64(version), HelloResponseCode, true,
		0, "Success.", hintsArray, hintResult.LastUpdateTime, hintResult.LastLiveQueryTime)
	body, err := controller.responseBodyEncoder.Encode(header)
	if err != nil {
		_ = c.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to encode response body: %w", err))
		return
	}
	utils.AddAdditionalLogFields(c, zap.Object("response-header", header))
	_, err = c.Writer.Write([]byte(body))
	if err != nil {
		_ = c.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to write response body: %w", err))
	}
}
