package controllers

import (
	"net/http"
	"net/http/httptest"

	"hello-api/internal/app/parsers/deviceresponseheader"

	"github.com/gin-gonic/gin"
	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"
)

var _ = Describe("prereg handler", func() {
	Context("bad header values", func() {
		It("Does not return maintenance mode on bad header values", func() {
			helloHandler := newPreregHandler(nil)
			recorder := httptest.NewRecorder()
			testContext, _ := gin.CreateTestContext(recorder)
			testContext.Request, _ = http.NewRequest("POST", "/hello", http.NoBody)
			for _, badHeaderValue := range []string{"potato", "mopar", "chevrolet", "3", "5", "13abd"} {
				testContext.Request.Header.Set(DeviceUnauthorizedHeader, badHeaderValue)
				helloHandler(testContext)
				Expect(recorder.Code).To(BeComparableTo(http.StatusOK))
				decoder := deviceresponseheader.NewResponseHeaderDecoder()
				_, err := decoder.Decode(recorder.Body.String())
				Expect(err).ToNot(BeNil())
			}
		})
	})
	It("returns 200 when Device-Unauthorized is set to 2 with maintenance mode response", func() {
		helloHandler := newPreregHandler(nil)
		recorder := httptest.NewRecorder()
		testContext, _ := gin.CreateTestContext(recorder)
		testContext.Request, _ = http.NewRequest("POST", "/hello", http.NoBody)
		testContext.Request.Header.Set(DeviceUnauthorizedHeader, preregHeaderValue)
		helloHandler(testContext)
		Expect(recorder.Code).To(BeComparableTo(http.StatusOK))
		decoder := deviceresponseheader.NewResponseHeaderDecoder()
		decodedBody, err := decoder.Decode(recorder.Body.String())
		Expect(err).To(BeNil())
		Expect(decodedBody.ErrorCode).To(BeEquivalentTo(backendMaintenanceModeErrroCode))
		Expect(decodedBody.Message).To(BeEquivalentTo(maintenanceModeMessage))
	})
})
