// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package middleware

import (
	"fmt"
	"net/http"

	"hello-api/internal/utils"

	"hello-api/internal/app/parsers/devicecontext"

	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

const deviceContextHeader = "Device-Context"

// DeviceContextMiddleware -- Gets the B64 encoded device context from the request header
func DeviceContextMiddleware(context *gin.Context) {
	contextHeader := context.GetHeader(deviceContextHeader)
	parser := devicecontext.NewDeviceAuthResponseParser()
	deviceAuthContext, err := parser.ParseResponse(contextHeader)
	if err != nil {
		_ = context.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to parse device context header: %w", err))
		return
	}
	utils.SetDeviceAuthContext(context, deviceAuthContext)
	utils.AddAdditionalLogFields(context, zap.Inline(deviceAuthContext))
	context.Next()
}
