// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package middleware

import (
	"errors"
	"fmt"
	"hello-api/internal/metrics"
	"net/http"
	"time"

	"hello-api/internal/utils"

	"hello-api/internal/lct"

	"carbonblack.com/protocols/psc/devicelastcontacttime"
	"github.com/gin-gonic/gin"
	"github.com/opentracing/opentracing-go"
)

func LctPublisherMiddleware(publisher lct.LctProducer, metricsClient metrics.Service) gin.HandlerFunc {
	handler := func(context *gin.Context) {
		func() {
			sp := opentracing.SpanFromContext(context.Request.Context())

			if sp != nil {
				childSp := opentracing.StartSpan("lct-publish", opentracing.ChildOf(sp.Context()))
				defer childSp.Finish()
			}

			deviceContext, err := utils.GetDeviceAuthContext(context)
			if err != nil {
				_ = context.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to get device context header: %w", err))
				return
			}
			if deviceContext == nil || deviceContext.Device == nil {
				_ = context.AbortWithError(http.StatusInternalServerError, errors.New("device context didn't include a valid device or was nil"))
				return
			}
			deviceLastContactTime := &devicelastcontacttime.DeviceLastContactTime{
				OrgKey:   deviceContext.Device.OrgKey,
				DeviceId: deviceContext.Device.GetId(), LastContactTimestamp: time.Now().UnixMilli(), DeregisterAfterTime: 0,
			}

			if err = publisher.Publish(deviceLastContactTime); err != nil {
				_ = context.AbortWithError(http.StatusInternalServerError, fmt.Errorf("failed to publish to last contact updater: %w", err))
				return
			}

			metricsClient.Count("lct_publish", nil, 1)
		}()

		context.Next()
	}
	return handler
}
