// Copyright (c) 2023 VMware Carbon Black, all rights reserved

// Package middleware contains classes dealing with execution middleware
package middleware

import (
	"runtime/debug"

	"hello-api/internal/config"
	"hello-api/internal/logger"
	"hello-api/internal/metrics"

	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

// MiddlewareUtils -- Structure used by the routing middleware
type MiddlewareUtils struct {
	RequestLogger logger.Logger
	cfg           *config.Config
	metricsClient metrics.Service
}

var RecoveryMiddleware = gin.CustomRecovery(func(c *gin.Context, err any) {
	l := logger.RequestLoggerFromContext(c.Request.Context())
	c.Set("error", "true")
	loggerErrors := make([]error, len(c.Errors))
	for i, ginError := range c.Errors {
		if ginError != nil {
			loggerErrors[i] = ginError.Err
		}
	}
	l.Error("Recovering from error", zap.Errors("context_errors", loggerErrors), zap.Any("error", err), zap.String("stacktrace", string(debug.Stack())))
})

func NewMiddlewareFactory(log logger.Logger, metricsClient metrics.Service, cfg *config.Config) *MiddlewareUtils {
	return &MiddlewareUtils{RequestLogger: log, cfg: cfg, metricsClient: metricsClient}
}
