// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package middleware

import (
	"net/http/pprof"

	"hello-api/internal/logger"

	"github.com/gin-gonic/gin"

	"hello-api/internal/config"
)

// RouteProfiler sets up profiler routes
func RouteProfiler(r *gin.Engine, cfg *config.Config) {
	r.GET(cfg.ServicePrefix+"/profiler/pprof/", index)
	r.GET(cfg.ServicePrefix+"/profiler/pprof/cmdline/", cmdline)
	r.GET(cfg.ServicePrefix+"/profiler/pprof/profile/", profile)
	r.GET(cfg.ServicePrefix+"/profiler/pprof/symbol/", symbol)
	r.GET(cfg.ServicePrefix+"/profiler/pprof/trace/", trace)

	r.GET(cfg.ServicePrefix+"/profiler/pprof/heap/", heap)
	r.GET(cfg.ServicePrefix+"/profiler/pprof/goroutine/", goroutine)
	r.GET(cfg.ServicePrefix+"/profiler/pprof/block/", block)
	r.GET(cfg.ServicePrefix+"/profiler/pprof/threadcreate/", threadcreate)
	r.GET(cfg.ServicePrefix+"/profiler/pprof/mutex/", mutex)
	logger.BaseLogger().Info("Profiler routes set up")
}

func index(ctx *gin.Context) {
	pprof.Index(ctx.Writer, ctx.Request)
}

func cmdline(ctx *gin.Context) {
	pprof.Cmdline(ctx.Writer, ctx.Request)
}

func profile(ctx *gin.Context) {
	pprof.Profile(ctx.Writer, ctx.Request)
}

func symbol(ctx *gin.Context) {
	pprof.Symbol(ctx.Writer, ctx.Request)
}

func trace(ctx *gin.Context) {
	pprof.Trace(ctx.Writer, ctx.Request)
}

func heap(ctx *gin.Context) {
	pprof.Handler("heap").ServeHTTP(ctx.Writer, ctx.Request)
}

func goroutine(ctx *gin.Context) {
	pprof.Handler("goroutine").ServeHTTP(ctx.Writer, ctx.Request)
}

func block(ctx *gin.Context) {
	pprof.Handler("block").ServeHTTP(ctx.Writer, ctx.Request)
}

func threadcreate(ctx *gin.Context) {
	pprof.Handler("threadcreate").ServeHTTP(ctx.Writer, ctx.Request)
}

func mutex(ctx *gin.Context) {
	pprof.Handler("mutex").ServeHTTP(ctx.Writer, ctx.Request)
}
