// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package middleware

import (
	"fmt"
	"net/http"
	"time"

	"hello-api/internal/logger"

	"github.com/AngelMarquez/timeout"
	"github.com/gin-gonic/gin"
)

// TimeoutMiddleware -- function to handle timeouts
func TimeoutMiddleware(timeoutDuration time.Duration) gin.HandlerFunc {
	return timeout.New(
		timeout.WithTimeout(timeoutDuration),
		timeout.WithHandler(func(c *gin.Context) {
			c.Next()
		}),
		timeout.WithResponse(func(c *gin.Context) {
			logger.RequestLoggerFromContext(c.Request.Context()).Error(fmt.Sprintf("Request timed out after %v ",
				timeoutDuration))
			c.Set("error", "true")
			c.String(http.StatusRequestTimeout, "Request took too long")
		}),
	)
}
