// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package config

import "sync"

// singleton configuration instance
var (
	onceLogger           sync.Once
	loggerConfigInstance *LoggerConfig
)

type LoggerConfig struct {
	LogLevel    string
	CommitSHA   string
	UseS3       bool
	S3Bucket    string
	S3ChunkSize int
}

func GetLoggerConfigInstance() *LoggerConfig {
	onceLogger.Do(func() {
		loggerConfigInstance = newLoggerDefaultConfig(true)
	})
	return loggerConfigInstance
}

func newLoggerDefaultConfig(loadFromEnvironment bool) *LoggerConfig {
	cfg := LoggerConfig{
		LogLevel:    NewConfigParamWithDefault(envVarKeyLogLevel, defaultLogLevel).Get(loadFromEnvironment),
		CommitSHA:   NewConfigParamWithDefault(envVarKeyCommitSHA, "").Get(loadFromEnvironment),
		UseS3:       NewBoolParam("LOG_S3_ENABLED", false).Get(loadFromEnvironment),
		S3Bucket:    NewConfigParamWithDefault("LOG_S3_BUCKET_NAME", "").Get(loadFromEnvironment),
		S3ChunkSize: NewIntParam("LOG_S3_CHUNK_SIZE", 1024*16).Get(loadFromEnvironment),
	}

	return &cfg
}
