package logger

import (
	"github.com/tannerryan/ring"
	"go.uber.org/zap"
)

var nopLogger = zap.NewNop()

const (
	maxSensorCount = 1000000
	errorRate      = 0.001
)

type FilteredLogger interface {
	Stop() error
	IfNotSeen([]byte) Logger
	Logger
}

type tester interface {
	Test([]byte) bool
	Add([]byte)
}

type bloomLogger struct {
	StoppableLogger
	tester
}

func (bl *bloomLogger) IfNotSeen(element []byte) Logger {
	if bl.Test(element) {
		return nopLogger
	}
	bl.Add(element)
	return bl.StoppableLogger
}

func newBloomedLogger(baseLogger StoppableLogger, test tester) FilteredLogger {
	return &bloomLogger{StoppableLogger: baseLogger, tester: test}
}

func NewBloomedLogger(baseLogger StoppableLogger) (FilteredLogger, error) {
	bloomFilter, err := ring.Init(maxSensorCount, errorRate)
	if err != nil {
		return nil, err
	}
	return newBloomedLogger(baseLogger, bloomFilter), err
}

type nopBloomedLogger struct {
	StoppableLogger
}

func (nop *nopBloomedLogger) IfNotSeen(element []byte) Logger {
	return nopLogger
}

func newNopBloomLogger() FilteredLogger {
	return &nopBloomedLogger{StoppableLogger: nopStopLogger(nopLogger)}
}
