// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package logger

import (
	"context"
)

// Level represents the level of a log message.
type Level int

type requestLoggerKey int

// ContextWithRequestLogger returns a context with the given ids stored
func ContextWithRequestLogger(ctx context.Context, l Logger) context.Context {
	var key requestLoggerKey
	return context.WithValue(ctx, key, l)
}

// RequestLoggerFromContext gets ids stored in a context
func RequestLoggerFromContext(ctx context.Context) Logger {
	var key requestLoggerKey
	l, ok := ctx.Value(key).(Logger)
	if !ok || l == nil {
		l = BaseLogger()
		l.Error("failed to fetch logger from context")
	}
	return l
}
