package logger

import (
	"fmt"

	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"
	"go.uber.org/zap"
)

var _ = Describe("s3 logger tests", func() {
	Context("stopabbleLogger struct", func() {
		It("calls the stop func when stopped", func() {
			called := false
			onStop := func() error {
				called = true
				return nil
			}
			testLogger := stopLoggerFunc(
				zap.NewNop(),
				onStop,
			)
			stopErr := testLogger.Stop()
			Expect(stopErr).To(BeNil())
			Expect(called).To(BeTrue())
		})
		It("returns the stopError when stopped", func() {
			called := false
			stopErr := fmt.Errorf("stopError")
			onStop := func() error {
				called = true
				return stopErr
			}
			testLogger := stopLoggerFunc(
				zap.NewNop(),
				onStop,
			)
			err := testLogger.Stop()
			Expect(err).To(MatchError(stopErr))
			Expect(called).To(BeTrue())
		})
	})
})
