package s3logger

import (
	"errors"
	"io"
)

var ErrLimitHit error = errors.New("writer limit exceeded")

// A write closer that automatically closes and returns a special error when the limit is overrun
type limitedWriteCloser struct {
	Writer  io.WriteCloser
	limit   int
	written int
}

func (u *limitedWriteCloser) Write(p []byte) (int, error) {
	n, err := u.Writer.Write(p)
	if err != nil {
		return n, err
	}
	u.written += n
	if u.written > u.limit {
		return n, ErrLimitHit
	}
	return n, err
}

func (u *limitedWriteCloser) Close() error {
	return u.Writer.Close()
}
