package s3logger

import (
	"context"
	"time"

	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"
)

type mockTaskSubmiter struct {
	submissions  []func()
	stopped      bool
	stopDuration time.Duration
}

func (ms *mockTaskSubmiter) Submit(task func()) {
	ms.submissions = append(ms.submissions, task)
}

func (ms *mockTaskSubmiter) StopAndWaitFor(duration time.Duration) {
	ms.stopped = true
	ms.stopDuration = duration
}

var _ = Describe("s3 uploader", func() {
	var testUploader *uploadTaskSubmitter
	var testMockTaskSubmiter *mockTaskSubmiter

	BeforeEach(func() {
		mockUploader := &mockS3Uploader{}
		testMockTaskSubmiter = &mockTaskSubmiter{}
		testUploader = newS3UploadSubmitter("bucketName", 10, testMockTaskSubmiter, mockUploader)
		Expect(testMockTaskSubmiter.stopped).To(BeFalse())
		Expect(testUploader.taskContext).ToNot(BeNil())
		Expect(testUploader.taskContext.Err()).To(BeNil())
	})

	It("uploads correctly", func() {
		testUploader.SubmitUpload()
		Expect(len(testMockTaskSubmiter.submissions)).To(BeEquivalentTo(1))
		Expect(testMockTaskSubmiter.stopped).To(BeFalse())
	})

	It("stops correctly", func() {
		testUploader.Stop()
		Expect(testMockTaskSubmiter.stopped).To(BeTrue())
		Expect(testMockTaskSubmiter.stopDuration).To(BeEquivalentTo(5 * time.Second))
		Expect(testUploader.taskContext.Err()).To(MatchError(context.Canceled))
	})
})
