// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package metrics

import (
	"testing"

	"hello-api/internal/metrics/wavefront"

	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"
	"hello-api/internal/config"
)

func TestRequestLogger(t *testing.T) {
	RegisterFailHandler(Fail)
	RunSpecs(t, "Logger")
}

var cfg = config.GetInstance()

var _ = Describe("Wavefront Reporter Unit tests", func() {
	cfg.WavefrontEndpoint = "http://localhost"

	// NOTE: No actual access to a wavefront server; all we can check is that basic plumbing works...

	Context("Creates a new request logger", func() {
		It("should create a valid logger with a commit SHA config and default level", func() {
			rep, err := wavefront.NewReporter(cfg)
			Expect(err).To(BeNil())
			Expect(rep).ToNot(BeNil())

			trc := rep.Tracer()
			Expect(trc).ToNot(BeNil())

			cfg := rep.GetConfig()
			Expect(cfg).ToNot(BeNil())

			rep.CloseConnection()
		})

		It("metrics usage", func() {
			rep, err := wavefront.NewReporter(cfg)
			Expect(err).To(BeNil())
			Expect(rep).ToNot(BeNil())

			tags := make(map[string]string)
			tags["shard"] = "shard"
			tags["msc"] = "msc2"
			rep.Gauge("test.shard.millis.gauge", tags, int64(23))
			rep.Histogram("test.shard.millis.histogram", tags, int64(23))
			rep.Count("test.shard.millis.count", tags, int64(23))
		})
	})
})
