// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package streamchecker

import (
	"context"
	"errors"
	"testing"

	"github.com/aws/aws-sdk-go-v2/service/kinesis"
	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"
)

const streamName = "STREAM"

// mockKinesisClient -- Structure use to feed the mock kinesis
type mockKinesisClient struct {
	streamDescriber
	streamExists bool
}

func (m *mockKinesisClient) DescribeStream(ctx context.Context, params *kinesis.DescribeStreamInput, optFns ...func(*kinesis.Options)) (*kinesis.DescribeStreamOutput, error) {
	if m.streamExists {
		return nil, nil
	}
	return nil, errors.New("STREAM DOESN'T EXIST")
}

func newMockKinesisClient(streamExists bool) *mockKinesisClient {
	return &mockKinesisClient{streamDescriber: nil, streamExists: streamExists}
}

func TestStreamChecker(t *testing.T) {
	RegisterFailHandler(Fail)
	RunSpecs(t, "stream checker tests")
}

var _ = Describe("stream checker", func() {
	It("reports when a stream exists", func() {
		mockClient := newMockKinesisClient(true)
		checker := NewStreamChecker(streamName, mockClient)
		Expect(checker.DoesStreamExist(context.TODO())).To(BeNil())
	})

	It("reports when a stream doesn't exist", func() {
		mockClient := newMockKinesisClient(false)
		checker := NewStreamChecker(streamName, mockClient)
		Expect(checker.DoesStreamExist(context.TODO())).ToNot(BeNil())
	})
})
